@@ -44,7 +44,6 @@ OVN_PORT_BINDING_PROFILE_PARAMS = [{'parent_name': six.string_types,
                                     'tag': six.integer_types},
                                    {'vtep-physical-switch': six.string_types,
                                     'vtep-logical-switch': six.string_types}]
-MIGRATING_ATTR = 'migrating_to'
 OVN_ROUTER_PORT_OPTION_KEYS = ['router-port', 'nat-addresses']
 OVN_GATEWAY_CHASSIS_KEY = 'redirect-chassis'
 OVN_GATEWAY_NAT_ADDRESSES_KEY = 'nat-addresses'
@@ -54,6 +53,11 @@ OVN_PROVNET_PORT_NAME_PREFIX = 'provnet-'
 
 OVN_NEUTRON_OWNER_TO_PORT_TYPE = {const.DEVICE_OWNER_DHCP: 'localport'}
 
+# Agent extension constants
+OVN_AGENT_DESC_KEY = 'neutron:description'
+OVN_CONTROLLER_AGENT = 'OVN Controller agent'
+OVN_CONTROLLER_GW_AGENT = 'OVN Controller Gateway agent'
+
 # OVN ACLs have priorities.  The highest priority ACL that matches is the one
 # that takes effect.  Our choice of priority numbers is arbitrary, but it
 # leaves room above and below the ACLs we create.  We only need two priorities.
