@@ -19,22 +19,18 @@ import threading
 from futurist import periodics
 from neutron.common import config as n_conf
 from neutron_lib import context as n_context
+from neutron_lib import exceptions as n_exc
 from neutron_lib import worker
-from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
 
 from networking_ovn.common import constants as ovn_const
 from networking_ovn.db import maintenance as db_maint
 from networking_ovn.db import revision as db_rev
-from networking_ovn import ovn_db_sync
 
-CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 DB_CONSISTENCY_CHECK_INTERVAL = 300  # 5 minutes
-INCONSISTENCY_TYPE_CREATE_UPDATE = 'create/update'
-INCONSISTENCY_TYPE_DELETE = 'delete'
 
 
 class MaintenanceWorker(worker.BaseWorker):
@@ -161,8 +157,16 @@ class DBInconsistenciesPeriodics(object):
     def _fix_create_update(self, row):
         res_map = self._resources_func_map[row.resource_type]
         admin_context = n_context.get_admin_context()
-        # Get the latest version of the resource in Neutron DB
-        n_obj = res_map['neutron_get'](admin_context, row.resource_uuid)
+        try:
+            # Get the latest version of the resource in Neutron DB
+            n_obj = res_map['neutron_get'](admin_context, row.resource_uuid)
+        except n_exc.NotFound:
+            LOG.warning('Skip fixing resource %(res_uuid)s (type: '
+                        '%(res_type)s). Resource does not exist in Neutron '
+                        'database anymore', {'res_uuid': row.resource_uuid,
+                                             'res_type': row.resource_type})
+            return
+
         ovn_obj = res_map['ovn_get'](row.resource_uuid)
 
         if not ovn_obj:
@@ -212,57 +216,6 @@ class DBInconsistenciesPeriodics(object):
         else:
             self._ovn_client.update_subnet(sn_db_obj, n_db_obj)
 
-    def _log_maintenance_inconsistencies(self, create_update_inconsistencies,
-                                         delete_inconsistencies):
-        if not CONF.debug:
-            return
-
-        def _log(inconsistencies, type_):
-            if not inconsistencies:
-                return
-
-            c = {}
-            for f in inconsistencies:
-                if f.resource_type not in c:
-                    c[f.resource_type] = 1
-                else:
-                    c[f.resource_type] += 1
-
-            fail_str = ', '.join('{}={}'.format(k, v) for k, v in c.items())
-            LOG.debug('Maintenance task: Number of inconsistencies '
-                      'found at %(type_)s: %(fail_str)s',
-                      {'type_': type_, 'fail_str': fail_str})
-
-        _log(create_update_inconsistencies, INCONSISTENCY_TYPE_CREATE_UPDATE)
-        _log(delete_inconsistencies, INCONSISTENCY_TYPE_DELETE)
-
-    # The migration will run just once per neutron-server instance. If the lock
-    # is held by some other neutron-server instance in the cloud, we'll attempt
-    # to perform the migration every 10 seconds until completed.
-    @periodics.periodic(spacing=10, run_immediately=True)
-    def migrate_to_port_groups(self):
-        """Perform the migration from Address Sets to Port Groups. """
-        # TODO(dalvarez): Remove this in U cycle when we're sure that all
-        # versions are running using Port Groups (and OVS >= 2.10).
-
-        # If Port Groups are not supported or we've already migrated, we don't
-        # need to attempt to migrate again.
-        if (not self._nb_idl.is_port_groups_supported() or
-                not self._nb_idl.get_address_sets()):
-            raise periodics.NeverAgain()
-
-        # Only the worker holding a valid lock within OVSDB will perform the
-        # migration.
-        if not self.has_lock:
-            return
-
-        admin_context = n_context.get_admin_context()
-        nb_sync = ovn_db_sync.OvnNbSynchronizer(
-            self._ovn_client._plugin, self._nb_idl, self._ovn_client._sb_idl,
-            None, None)
-        nb_sync.migrate_to_port_groups(admin_context)
-        raise periodics.NeverAgain()
-
     @periodics.periodic(spacing=DB_CONSISTENCY_CHECK_INTERVAL,
                         run_immediately=True)
     def check_for_inconsistencies(self):
@@ -274,22 +227,13 @@ class DBInconsistenciesPeriodics(object):
         create_update_inconsistencies = db_maint.get_inconsistent_resources()
         delete_inconsistencies = db_maint.get_deleted_resources()
         if not any([create_update_inconsistencies, delete_inconsistencies]):
-            LOG.debug('Maintenance task: No inconsistencies found. Skipping')
             return
-
         LOG.debug('Maintenance task: Synchronizing Neutron '
                   'and OVN databases')
-        self._log_maintenance_inconsistencies(create_update_inconsistencies,
-                                              delete_inconsistencies)
         self._sync_timer.restart()
 
-        dbg_log_msg = ('Maintenance task: Fixing resource %(res_uuid)s '
-                       '(type: %(res_type)s) at %(type_)s')
         # Fix the create/update resources inconsistencies
         for row in create_update_inconsistencies:
-            LOG.debug(dbg_log_msg, {'res_uuid': row.resource_uuid,
-                                    'res_type': row.resource_type,
-                                    'type_': INCONSISTENCY_TYPE_CREATE_UPDATE})
             try:
                 # NOTE(lucasagomes): The way to fix subnets is bit
                 # different than other resources. A subnet in OVN language
@@ -302,30 +246,27 @@ class DBInconsistenciesPeriodics(object):
                 else:
                     self._fix_create_update(row)
             except Exception:
-                LOG.exception('Maintenance task: Failed to fix resource '
-                              '%(res_uuid)s (type: %(res_type)s)',
+                LOG.exception('Failed to fix resource %(res_uuid)s '
+                              '(type: %(res_type)s)',
                               {'res_uuid': row.resource_uuid,
                                'res_type': row.resource_type})
 
         # Fix the deleted resources inconsistencies
         for row in delete_inconsistencies:
-            LOG.debug(dbg_log_msg, {'res_uuid': row.resource_uuid,
-                                    'res_type': row.resource_type,
-                                    'type_': INCONSISTENCY_TYPE_DELETE})
             try:
                 if row.resource_type == ovn_const.TYPE_SUBNETS:
                     self._ovn_client.delete_subnet(row.resource_uuid)
                 else:
                     self._fix_delete(row)
             except Exception:
-                LOG.exception('Maintenance task: Failed to fix deleted '
-                              'resource %(res_uuid)s (type: %(res_type)s)',
+                LOG.exception('Failed to fix deleted resource %(res_uuid)s '
+                              '(type: %(res_type)s)',
                               {'res_uuid': row.resource_uuid,
                                'res_type': row.resource_type})
 
-        LOG.info('Maintenance task: Synchronization finished '
-                 '(took %.2f seconds)', self._sync_timer.elapsed())
         self._sync_timer.stop()
+        LOG.info('Maintenance task synchronization finished '
+                 '(took %.2f seconds)', self._sync_timer.elapsed())
 
     def _create_lrouter_port(self, port):
         admin_context = n_context.get_admin_context()
