@@ -17,7 +17,6 @@ import collections
 import copy
 
 import netaddr
-from neutron.plugins.common import utils as p_utils
 from neutron_lib.api.definitions import l3
 from neutron_lib.api.definitions import port_security as psec
 from neutron_lib.api.definitions import portbindings
@@ -27,6 +26,7 @@ from neutron_lib import context as n_context
 from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
+from neutron_lib.plugins import utils as p_utils
 from neutron_lib.utils import helpers
 from neutron_lib.utils import net as n_net
 from oslo_config import cfg
@@ -53,8 +53,7 @@ OvnPortInfo = collections.namedtuple(
 
 
 GW_INFO = collections.namedtuple('GatewayInfo', ['network_id', 'subnet_id',
-                                                 'router_ip', 'gateway_ip',
-                                                 'ip_version', 'ip_prefix'])
+                                                 'router_ip', 'gateway_ip'])
 
 
 class OVNClient(object):
@@ -284,32 +283,19 @@ class OVNClient(object):
             # The lport_name *must* be neutron port['id'].  It must match the
             # iface-id set in the Interfaces table of the Open_vSwitch
             # database which nova sets to be the port ID.
-
-            kwargs = {
-                'lport_name': port['id'],
-                'lswitch_name': lswitch_name,
-                'addresses': port_info.addresses,
-                'external_ids': external_ids,
-                'parent_name': port_info.parent_name,
-                'tag': port_info.tag,
-                'enabled': port.get('admin_state_up'),
-                'options': port_info.options,
-                'type': port_info.type,
-                'port_security': port_info.port_security,
-                'dhcpv4_options': dhcpv4_options,
-                'dhcpv6_options': dhcpv6_options
-            }
-
-            # NOTE(mjozefcz): Do not set addresses if the port is not
-            # bound and has no device_owner - possibly it is a VirtualIP
-            # port used for Octavia (VRRP).
-            # For more details check related bug #1789686.
-            if (not port.get('device_owner') and
-                port.get(portbindings.VIF_TYPE) ==
-                    portbindings.VIF_TYPE_UNBOUND):
-                kwargs['addresses'] = []
-
-            port_cmd = txn.add(self._nb_idl.create_lswitch_port(**kwargs))
+            port_cmd = txn.add(self._nb_idl.create_lswitch_port(
+                lport_name=port['id'],
+                lswitch_name=lswitch_name,
+                addresses=port_info.addresses,
+                external_ids=external_ids,
+                parent_name=port_info.parent_name,
+                tag=port_info.tag,
+                enabled=port.get('admin_state_up'),
+                options=port_info.options,
+                type=port_info.type,
+                port_security=port_info.port_security,
+                dhcpv4_options=dhcpv4_options,
+                dhcpv6_options=dhcpv6_options))
 
             # Handle ACL's for this port. If we're not using Port Groups
             # because either the schema doesn't support it or we didn't
@@ -329,7 +315,7 @@ class OVNClient(object):
                     txn.add(self._nb_idl.pg_add_ports(
                         utils.ovn_port_group_name(sg), port_cmd))
             else:
-                # SGs modelled as Address Sets:
+                # SGs modelled as Port Groups:
                 acls_new = ovn_acl.add_acls(self._plugin, admin_context,
                                             port, sg_cache, subnet_cache,
                                             self._nb_idl)
@@ -414,16 +400,6 @@ class OVNClient(object):
                 dhcpv6_options = txn.add(port_info.dhcpv6_options['cmd'])
             else:
                 dhcpv6_options = [port_info.dhcpv6_options['uuid']]
-
-            # NOTE(mjozefcz): Do not set addresses if the port is not
-            # bound and has no device_owner - possibly it is a VirtualIP
-            # port used for Octavia (VRRP).
-            # For more details check related bug #1789686.
-            if (not port.get('device_owner') and
-                port.get(portbindings.VIF_TYPE) ==
-                    portbindings.VIF_TYPE_UNBOUND):
-                columns_dict['addresses'] = []
-
             # NOTE(lizk): Fail port updating if port doesn't exist. This
             # prevents any new inserted resources to be orphan, such as port
             # dhcp options or ACL rules for port, e.g. a port was created
@@ -477,11 +453,9 @@ class OVNClient(object):
                         utils.is_port_security_enabled(port)):
                     self._add_port_to_drop_port_group(port['id'], txn)
                 # If the port doesn't belong to any security group and
-                # port_security is disabled, or it's a trusted port, then
-                # allow all traffic.
-                elif ((not new_sg_ids and
-                      not utils.is_port_security_enabled(port)) or
-                      utils.is_lsp_trusted(port)):
+                # port_security is disabled, allow all traffic
+                elif (not new_sg_ids and
+                      not utils.is_port_security_enabled(port)):
                     self._del_port_from_drop_port_group(port['id'], txn)
             else:
                 # Refresh ACLs for changed security groups or fixed IPs.
@@ -608,7 +582,9 @@ class OVNClient(object):
         context = n_context.get_admin_context()
         fip_db = self._l3_plugin._get_floatingip(context, floatingip['id'])
 
+        func = self._nb_idl.add_nat_rule_in_lrouter
         gw_lrouter_name = utils.ovn_name(router_id)
+        nat_rule_args = (gw_lrouter_name,)
         # TODO(chandrav): Since the floating ip port is not
         # bound to any chassis, packets destined to floating ip
         # will be dropped. To overcome this, delete the floating
@@ -619,6 +595,21 @@ class OVNClient(object):
                         fip_db['floating_port_id'],
                         utils.ovn_name(floatingip['floating_network_id'])))
 
+        # Get the list of nat rules and check if the external_ip
+        # with type 'dnat_and_snat' already exists or not.
+        # If exists, set the new value.
+        # This happens when the port associated to a floating ip
+        # is deleted before the disassociation.
+        lrouter_nat_rules = self._nb_idl.get_lrouter_nat_rules(
+            gw_lrouter_name)
+        for nat_rule in lrouter_nat_rules:
+            if (nat_rule['external_ip'] ==
+                    floatingip['floating_ip_address'] and
+                    nat_rule['type'] == 'dnat_and_snat'):
+                func = self._nb_idl.set_nat_rule_in_lrouter
+                nat_rule_args = (gw_lrouter_name, nat_rule['uuid'])
+                break
+
         ext_ids = {
             ovn_const.OVN_FIP_EXT_ID_KEY: floatingip['id'],
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(utils.get_revision_number(
@@ -635,19 +626,14 @@ class OVNClient(object):
             columns['logical_port'] = floatingip['port_id']
             ext_ids[ovn_const.OVN_FIP_EXT_MAC_KEY] = port['mac_address']
             if self._nb_idl.lsp_get_up(floatingip['port_id']).execute():
-                port_net = self._plugin.get_network(context,
-                                                    port['network_id'])
-                if port_net.get(pnet.NETWORK_TYPE) != const.TYPE_VLAN:
-                    columns['external_mac'] = port['mac_address']
+                columns['external_mac'] = port['mac_address']
 
         # TODO(dalvarez): remove this check once the minimum OVS required
         # version contains the column (when OVS 2.8.2 is released).
         if self._nb_idl.is_col_present('NAT', 'external_ids'):
             columns['external_ids'] = ext_ids
 
-        commands.append(self._nb_idl.add_nat_rule_in_lrouter(gw_lrouter_name,
-                                                             **columns))
-
+        commands.append(func(*nat_rule_args, **columns))
         self._transaction(commands, txn=txn)
 
     def _delete_floatingip(self, fip, lrouter, txn=None):
@@ -764,18 +750,16 @@ class OVNClient(object):
                           'router. Error: %s', e)
 
     def _get_gw_info(self, context, router):
-        gateways_info = []
         ext_gw_info = router.get(l3.EXTERNAL_GW_INFO, {})
         network_id = ext_gw_info.get('network_id', '')
         for ext_fixed_ip in ext_gw_info.get('external_fixed_ips', []):
             subnet_id = ext_fixed_ip['subnet_id']
             subnet = self._plugin.get_subnet(context, subnet_id)
-            gateways_info.append(GW_INFO(
-                network_id, subnet_id, ext_fixed_ip['ip_address'],
-                subnet.get('gateway_ip'), subnet['ip_version'],
-                const.IPv4_ANY if subnet['ip_version'] == const.IP_VERSION_4
-                else const.IPv6_ANY))
-        return gateways_info
+            if subnet['ip_version'] == 4:
+                return GW_INFO(
+                    network_id, subnet_id, ext_fixed_ip['ip_address'],
+                    subnet.get('gateway_ip'))
+        return GW_INFO('', '', '', '')
 
     def _delete_router_ext_gw(self, context, router, networks, txn):
         if not networks:
@@ -783,19 +767,17 @@ class OVNClient(object):
         router_id = router['id']
         gw_port_id = router['gw_port_id']
         gw_lrouter_name = utils.ovn_name(router_id)
-        gateways = self._get_gw_info(context, router)
-        for gw_info in gateways:
-            if gw_info.ip_version == const.IP_VERSION_4:
-                for network in networks:
-                    txn.add(self._nb_idl.delete_nat_rule_in_lrouter(
-                        gw_lrouter_name, type='snat', logical_ip=network,
-                        external_ip=gw_info.router_ip))
-            txn.add(self._nb_idl.delete_static_route(
-                gw_lrouter_name, ip_prefix=gw_info.ip_prefix,
-                nexthop=gw_info.gateway_ip))
+        gw_info = self._get_gw_info(context, router)
+        txn.add(self._nb_idl.delete_static_route(gw_lrouter_name,
+                                                 ip_prefix='0.0.0.0/0',
+                                                 nexthop=gw_info.gateway_ip))
         txn.add(self._nb_idl.delete_lrouter_port(
             utils.ovn_lrouter_port_name(gw_port_id),
             gw_lrouter_name))
+        for network in networks:
+            txn.add(self._nb_idl.delete_nat_rule_in_lrouter(
+                gw_lrouter_name, type='snat', logical_ip=network,
+                external_ip=gw_info.router_ip))
 
     def _get_nets_and_ipv6_ra_confs_for_router_port(
             self, port_fixed_ips):
@@ -826,55 +808,45 @@ class OVNClient(object):
     def _add_router_ext_gw(self, context, router, networks, txn):
         router_id = router['id']
         # 1. Add the external gateway router port.
-        gateways = self._get_gw_info(context, router)
+        gw_info = self._get_gw_info(context, router)
         gw_port_id = router['gw_port_id']
         port = self._plugin.get_port(context, gw_port_id)
         self._create_lrouter_port(router_id, port, txn=txn)
 
-        def _build_extids(gw_info):
-            # TODO(lucasagomes): Remove this check after OVS 2.8.2 is tagged
-            # (prior to that, the external_ids column didn't exist in this
-            # table).
-            columns = {}
-            if self._nb_idl.is_col_present('Logical_Router_Static_Route',
-                                           'external_ids'):
-                columns['external_ids'] = {
-                    ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
-                    ovn_const.OVN_SUBNET_EXT_ID_KEY: gw_info.subnet_id}
-            return columns
+        # TODO(lucasagomes): Remove this check after OVS 2.8.2 is tagged
+        # (prior to that, the external_ids column didn't exist in this
+        # table).
+        columns = {}
+        if self._nb_idl.is_col_present('Logical_Router_Static_Route',
+                                       'external_ids'):
+            columns['external_ids'] = {
+                ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
+                ovn_const.OVN_SUBNET_EXT_ID_KEY: gw_info.subnet_id}
 
         # 2. Add default route with nexthop as gateway ip
         lrouter_name = utils.ovn_name(router_id)
-        for gw_info in gateways:
-            columns = _build_extids(gw_info)
-            txn.add(self._nb_idl.add_static_route(
-                lrouter_name, ip_prefix=gw_info.ip_prefix,
-                nexthop=gw_info.gateway_ip, **columns))
+        txn.add(self._nb_idl.add_static_route(
+            lrouter_name, ip_prefix='0.0.0.0/0', nexthop=gw_info.gateway_ip,
+            **columns))
 
         # 3. Add snat rules for tenant networks in lrouter if snat is enabled
         if utils.is_snat_enabled(router) and networks:
             self.update_nat_rules(router, networks, enable_snat=True, txn=txn)
         return port
 
-    def _check_external_ips_changed(self, context, ovn_snats,
-                                    ovn_static_routes, router):
-        gateways = self._get_gw_info(context, router)
-        ovn_gw_subnets = None
+    def _check_external_ips_changed(self, context, ovn_snats, ovn_static_route,
+                                    router):
+        gw_info = self._get_gw_info(context, router)
         if self._nb_idl.is_col_present('Logical_Router_Static_Route',
                                        'external_ids'):
-            ovn_gw_subnets = [
-                getattr(route, 'external_ids', {}).get(
-                    ovn_const.OVN_SUBNET_EXT_ID_KEY) for route in
-                ovn_static_routes]
+            ext_ids = getattr(ovn_static_route, 'external_ids', {})
+            if (ext_ids.get(ovn_const.OVN_SUBNET_EXT_ID_KEY) !=
+               gw_info.subnet_id):
+                return True
 
-        for gw_info in gateways:
-            if ovn_gw_subnets and gw_info.subnet_id not in ovn_gw_subnets:
+        for snat in ovn_snats:
+            if snat.external_ip != gw_info.router_ip:
                 return True
-            if gw_info.ip_version == 6:
-                continue
-            for snat in ovn_snats:
-                if snat.external_ip != gw_info.router_ip:
-                    return True
 
         return False
 
@@ -1318,12 +1290,12 @@ class OVNClient(object):
         func = (self._nb_idl.add_nat_rule_in_lrouter if enable_snat else
                 self._nb_idl.delete_nat_rule_in_lrouter)
         gw_lrouter_name = utils.ovn_name(router['id'])
-        gateways = self._get_gw_info(context, router)
-        # Update NAT rules only for IPv4 subnets
-        commands = [func(gw_lrouter_name, type='snat', logical_ip=network,
-                         external_ip=gw_info.router_ip) for gw_info in gateways
-                    if gw_info.ip_version != const.IP_VERSION_6
-                    for network in networks]
+        gw_info = self._get_gw_info(context, router)
+        commands = []
+        for network in networks:
+            commands.append(
+                func(gw_lrouter_name, type='snat', logical_ip=network,
+                     external_ip=gw_info.router_ip))
         self._transaction(commands, txn=txn)
 
     def _create_provnet_port(self, txn, network, physnet, tag):
@@ -1649,9 +1621,31 @@ class OVNClient(object):
             if subnet['ip_version'] == 4:
                 context = n_context.get_admin_context()
                 self.update_metadata_port(context, network['id'])
+
             self._add_subnet_dhcp_options(subnet, network)
+
+            if self._nb_idl.is_port_groups_supported():
+                with self._nb_idl.transaction(check_error=True) as txn:
+                    self._create_subnet_port_group(subnet, txn)
+
         db_rev.bump_revision(subnet, ovn_const.TYPE_SUBNETS)
 
+    def _create_subnet_port_group(self, subnet, txn):
+        pg_name = utils.ovn_port_group_name(subnet['id'])
+        if self._nb_idl.get_port_group(pg_name):
+            return
+        # Create subnet Port Group.
+        txn.add(self._nb_idl.pg_add(pg_name, acls=[]))
+        # Add ACLs to this Port Group.
+        acls = ovn_acl.add_acls_for_subnet_port_group(
+            self._nb_idl, pg_name, subnet)
+        for acl in acls:
+            txn.add(self._nb_idl.pg_acl_add(**acl))
+
+    def _delete_subnet_port_group(self, subnet_id, txn):
+        pg_name = utils.ovn_port_group_name(subnet_id)
+        txn.add(self._nb_idl.pg_del(pg_name, if_exists=True))
+
     def update_subnet(self, subnet, network):
         ovn_subnet = self._nb_idl.get_subnet_dhcp_options(
             subnet['id'])['subnet']
@@ -1666,8 +1660,12 @@ class OVNClient(object):
             txn.add(check_rev_cmd)
             if subnet['enable_dhcp'] and not ovn_subnet:
                 self._enable_subnet_dhcp_options(subnet, network, txn)
+                if self._nb_idl.is_port_groups_supported():
+                    self._create_subnet_port_group(subnet, txn)
             elif not subnet['enable_dhcp'] and ovn_subnet:
                 self._remove_subnet_dhcp_options(subnet['id'], txn)
+                if self._nb_idl.is_port_groups_supported():
+                    self._delete_subnet_port_group(subnet, txn)
             elif subnet['enable_dhcp'] and ovn_subnet:
                 self._update_subnet_dhcp_options(subnet, network, txn)
 
@@ -1677,6 +1675,8 @@ class OVNClient(object):
     def delete_subnet(self, subnet_id):
         with self._nb_idl.transaction(check_error=True) as txn:
             self._remove_subnet_dhcp_options(subnet_id, txn)
+            if self._nb_idl.is_port_groups_supported():
+                self._delete_subnet_port_group(subnet_id, txn)
         db_rev.delete_revision(subnet_id, ovn_const.TYPE_SUBNETS)
 
     def create_security_group(self, security_group):
@@ -1707,11 +1707,11 @@ class OVNClient(object):
         with self._nb_idl.transaction(check_error=True) as txn:
             if not self._nb_idl.get_port_group(pg_name):
                 # If drop Port Group doesn't exist yet, create it.
-                txn.add(self._nb_idl.pg_add(pg_name, acls=[], may_exist=True))
+                txn.add(self._nb_idl.pg_add(pg_name, acls=[]))
                 # Add ACLs to this Port Group so that all traffic is dropped.
                 acls = ovn_acl.add_acls_for_drop_port_group(pg_name)
                 for acl in acls:
-                    txn.add(self._nb_idl.pg_acl_add(may_exist=True, **acl))
+                    txn.add(self._nb_idl.pg_acl_add(**acl))
 
             if ports:
                 ports_ids = [port['id'] for port in ports]
@@ -1788,6 +1788,7 @@ class OVNClient(object):
                         {'network_id': network['id'],
                          'tenant_id': network['project_id'],
                          'device_owner': const.DEVICE_OWNER_DHCP}}
+                # TODO(boden): rehome create_port into neutron-lib
                 p_utils.create_port(self._plugin, context, port)
             elif len(metadata_ports) > 1:
                 LOG.error("More than one metadata ports found for network %s. "
