@@ -13,7 +13,6 @@
 #
 
 from neutron_lib.api.definitions import external_net
-from neutron_lib.api.definitions import l3
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net as pnet
 from neutron_lib.callbacks import events
@@ -160,115 +159,58 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                 super(OVNL3RouterPlugin, self).create_router(
                     context, {'router': original_router})
 
-    def add_router_interface(self, context, router_id, interface_info):
-        router_interface_info = \
-            super(OVNL3RouterPlugin, self).add_router_interface(
-                context, router_id, interface_info)
-        port = self._plugin.get_port(context, router_interface_info['port_id'])
+    def _add_neutron_router_interface(self, context, router_id,
+                                      interface_info, may_exist=False):
+        try:
+            router_interface_info = (
+                super(OVNL3RouterPlugin, self).add_router_interface(
+                    context, router_id, interface_info))
+        except n_exc.PortInUse:
+            if not may_exist:
+                raise
+            # NOTE(lucasagomes): If the port is already being used it means
+            # the interface has been created already, let's just fetch it from
+            # the database. Perhaps the code below should live in Neutron
+            # itself, a get_router_interface() method in the main class
+            # would be handy
+            port = self._plugin.get_port(context, interface_info['port_id'])
+            subnets = [self._plugin.get_subnet(context, s)
+                       for s in utils.get_port_subnet_ids(port)]
+            router_interface_info = (
+                self._make_router_interface_info(
+                    router_id, port['tenant_id'], port['id'],
+                    port['network_id'], subnets[0]['id'],
+                    [subnet['id'] for subnet in subnets]))
 
-        multi_prefix = False
-        if (len(router_interface_info['subnet_ids']) == 1 and
-                len(port['fixed_ips']) > 1):
-            # NOTE(lizk) It's adding a subnet onto an already existing router
-            # interface port, try to update lrouter port 'networks' column.
-            self._ovn_client.update_router_port(port,
-                                                bump_db_rev=False)
-            multi_prefix = True
-        else:
-            self._ovn_client.create_router_port(router_id, port)
+        return router_interface_info
 
-        router = self.get_router(context, router_id)
-        if not router.get(l3.EXTERNAL_GW_INFO):
-            db_rev.bump_revision(port, ovn_const.TYPE_ROUTER_PORTS)
-            return router_interface_info
+    def add_router_interface(self, context, router_id, interface_info,
+                             may_exist=False):
+        router_interface_info = self._add_neutron_router_interface(
+            context, router_id, interface_info, may_exist=may_exist)
+        try:
+            self._ovn_client.create_router_port(router_id,
+                                                router_interface_info)
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                super(OVNL3RouterPlugin, self).remove_router_interface(
+                    context, router_id, router_interface_info)
 
-        cidr = None
-        for fixed_ip in port['fixed_ips']:
-            subnet = self._plugin.get_subnet(context, fixed_ip['subnet_id'])
-            if multi_prefix:
-                if 'subnet_id' in interface_info:
-                    if subnet['id'] is not interface_info['subnet_id']:
-                        continue
-            if subnet['ip_version'] == 4:
-                cidr = subnet['cidr']
-
-        if utils.is_snat_enabled(router) and cidr:
-            try:
-                self._ovn_client.update_nat_rules(router, networks=[cidr],
-                                                  enable_snat=True)
-            except Exception:
-                with excutils.save_and_reraise_exception():
-                    self._ovn.delete_lrouter_port(
-                        utils.ovn_lrouter_port_name(port['id']),
-                        utils.ovn_name(router_id)).execute(check_error=True)
-                    super(OVNL3RouterPlugin, self).remove_router_interface(
-                        context, router_id, router_interface_info)
-                    LOG.error('Error updating snat for subnet %(subnet)s in '
-                              'router %(router)s',
-                              {'subnet': router_interface_info['subnet_id'],
-                               'router': router_id})
-
-        db_rev.bump_revision(port, ovn_const.TYPE_ROUTER_PORTS)
         return router_interface_info
 
     def remove_router_interface(self, context, router_id, interface_info):
         router_interface_info = \
             super(OVNL3RouterPlugin, self).remove_router_interface(
                 context, router_id, interface_info)
-        router = self.get_router(context, router_id)
-        port_id = router_interface_info['port_id']
-        multi_prefix = False
-        port_removed = False
         try:
-            port = self._plugin.get_port(context, port_id)
-            # The router interface port still exists, call ovn to update it.
-            self._ovn_client.update_router_port(port,
-                                                bump_db_rev=False)
-            multi_prefix = True
-        except n_exc.PortNotFound:
-            # The router interface port doesn't exist any more,
-            # we will call ovn to delete it once we remove the snat
-            # rules in the router itself if we have to
-            port_removed = True
-
-        if not router.get(l3.EXTERNAL_GW_INFO):
-            if port_removed:
-                self._ovn_client.delete_router_port(port_id, router_id)
-            return router_interface_info
-
-        try:
-            cidr = None
-            if multi_prefix:
-                subnet = self._plugin.get_subnet(context,
-                                                 interface_info['subnet_id'])
-                if subnet['ip_version'] == 4:
-                    cidr = subnet['cidr']
-            else:
-                subnet_ids = router_interface_info.get('subnet_ids')
-                for subnet_id in subnet_ids:
-                    subnet = self._plugin.get_subnet(context, subnet_id)
-                    if subnet['ip_version'] == 4:
-                        cidr = subnet['cidr']
-                        break
-
-            if utils.is_snat_enabled(router) and cidr:
-                self._ovn_client.update_nat_rules(
-                    router, networks=[cidr], enable_snat=False)
+            port_id = router_interface_info['port_id']
+            subnet_ids = router_interface_info.get('subnet_ids')
+            self._ovn_client.delete_router_port(port_id, router_id=router_id,
+                                                subnet_ids=subnet_ids)
         except Exception:
             with excutils.save_and_reraise_exception():
                 super(OVNL3RouterPlugin, self).add_router_interface(
                     context, router_id, interface_info)
-                LOG.error('Error is deleting snat')
-
-        # NOTE(mangelajo): If the port doesn't exist anymore, we delete the
-        # router port as the last operation and update the revision database
-        # to ensure consistency
-        if port_removed:
-            self._ovn_client.delete_router_port(port_id, router_id)
-        else:
-            # otherwise, we just update the revision database
-            db_rev.bump_revision(port, ovn_const.TYPE_ROUTER_PORTS)
-
         return router_interface_info
 
     def create_floatingip_precommit(self, resource, event, trigger, context,
@@ -351,12 +293,25 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         return port_physnet_dict
 
     def update_router_gateway_port_bindings(self, router, host):
+        status = (n_const.PORT_STATUS_ACTIVE if host
+                  else n_const.PORT_STATUS_DOWN)
         context = n_context.get_admin_context()
         filters = {'device_id': [router],
                    'device_owner': [n_const.DEVICE_OWNER_ROUTER_GW]}
         for port in self._plugin.get_ports(context, filters=filters):
-            self._plugin.update_port(
-                context, port['id'], {'port': {portbindings.HOST_ID: host}})
+            # FIXME(lucasagomes): Ideally here we would use only
+            # one database transaction for the status and binding the
+            # host but, even tho update_port_status() receives a "host"
+            # parameter apparently it doesn't work for ports which the
+            # device owner is router_gateway. We need to look into it and
+            # fix the problem in Neutron before updating it here.
+            if host:
+                self._plugin.update_port(
+                    context, port['id'],
+                    {'port': {portbindings.HOST_ID: host}})
+
+            if port['status'] != status:
+                self._plugin.update_port_status(context, port['id'], status)
 
     def schedule_unhosted_gateways(self):
         port_physnet_dict = self._get_gateway_port_physnet_mapping()
@@ -364,13 +319,15 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         cms = self._sb_ovn.get_gateway_chassis_from_cms_options()
         unhosted_gateways = self._ovn.get_unhosted_gateways(
             port_physnet_dict, chassis_physnets, cms)
-        with self._ovn.transaction(check_error=True) as txn:
-            for g_name in unhosted_gateways:
-                physnet = port_physnet_dict.get(g_name[len('lrp-'):])
-                candidates = self._ovn_client.get_candidates_for_scheduling(
-                    physnet, cms=cms, chassis_physnets=chassis_physnets)
-                chassis = self.scheduler.select(
-                    self._ovn, self._sb_ovn, g_name, candidates=candidates)
+        for g_name in unhosted_gateways:
+            physnet = port_physnet_dict.get(g_name[len('lrp-'):])
+            candidates = self._ovn_client.get_candidates_for_scheduling(
+                physnet, cms=cms, chassis_physnets=chassis_physnets)
+            chassis = self.scheduler.select(
+                self._ovn, self._sb_ovn, g_name, candidates=candidates)
+            # NOTE(dalvarez): Let's commit the changes in separate transactions
+            # as we will rely on those for scheduling subsequent gateways.
+            with self._ovn.transaction(check_error=True) as txn:
                 txn.add(self._ovn.update_lrouter_port(
                     g_name, gateway_chassis=chassis))
 
