@@ -12,8 +12,12 @@
 #    under the License.
 #
 
-import copy
+import datetime
+import functools
+import operator
 import threading
+import types
+import uuid
 
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
@@ -31,7 +35,6 @@ from oslo_log import log
 
 from neutron.common import utils as n_utils
 from neutron.db import provisioning_blocks
-from neutron.plugins.ml2 import db as ml2_db
 from neutron.services.segments import db as segment_service_db
 
 from networking_ovn._i18n import _
@@ -133,7 +136,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             portbindings.VIF_TYPE_VHOST_USER: {
                 portbindings.CAP_PORT_FILTER: False,
                 portbindings.VHOST_USER_MODE:
-                portbindings.VHOST_USER_MODE_SERVER,
+                portbindings.VHOST_USER_MODE_CLIENT,
                 portbindings.VHOST_USER_OVS_PLUG: True
             }
         }
@@ -178,6 +181,12 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._ovn_client_inst = None
         self._nb_ovn, self._sb_ovn = impl_idl_ovn.get_ovn_idls(self,
                                                                trigger)
+        # Override agents API methods
+        self.patch_plugin_merge("get_agents", get_agents)
+        self.patch_plugin_choose("get_agent", get_agent)
+        self.patch_plugin_choose("update_agent", update_agent)
+        self.patch_plugin_choose("delete_agent", delete_agent)
+
         # Now IDL connections can be safely used.
         self._post_fork_event.set()
 
@@ -383,8 +392,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             return
         utils.validate_and_get_data_from_binding_profile(port)
         if self._is_port_provisioning_required(port, context.host):
-            self._insert_port_provisioning_block(context._plugin_context,
-                                                 port['id'])
+            self._insert_port_provisioning_block(context._plugin_context, port)
 
         db_rev.create_initial_revision(port['id'],
                                        ovn_const.TYPE_PORTS,
@@ -428,12 +436,13 @@ class OVNMechanismDriver(api.MechanismDriver):
 
         return True
 
-    def _insert_port_provisioning_block(self, context, port_id):
+    def _insert_port_provisioning_block(self, context, port):
         # Insert a provisioning block to prevent the port from
         # transitioning to active until OVN reports back that
         # the port is up.
         provisioning_blocks.add_provisioning_component(
-            context, port_id, resources.PORT,
+            context,
+            port['id'], resources.PORT,
             provisioning_blocks.L2_AGENT_ENTITY
         )
 
@@ -472,7 +481,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         drastically affect performance.  Raising an exception will
         result in the deletion of the resource.
         """
-        port = copy.deepcopy(context.current)
+        port = context.current
         port['network'] = context.network.current
         self._ovn_client.create_port(port)
         self._notify_dhcp_updated(port['id'])
@@ -498,8 +507,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         utils.validate_and_get_data_from_binding_profile(port)
         if self._is_port_provisioning_required(port, context.host,
                                                context.original_host):
-            self._insert_port_provisioning_block(context._plugin_context,
-                                                 port['id'])
+            self._insert_port_provisioning_block(context._plugin_context, port)
 
         if utils.is_lsp_router_port(port):
             # handle the case when an existing port is added to a
@@ -525,30 +533,10 @@ class OVNMechanismDriver(api.MechanismDriver):
         state. It is up to the mechanism driver to ignore state or
         state changes that it does not know or care about.
         """
-        port = copy.deepcopy(context.current)
+        port = context.current
         port['network'] = context.network.current
-        original_port = copy.deepcopy(context.original)
+        original_port = context.original
         original_port['network'] = context.network.current
-
-        # NOTE(mjozefcz): Check if port is in migration state. If so update
-        # the port status from DOWN to UP in order to generate 'fake'
-        # vif-interface-plugged event. This workaround is needed to
-        # perform live-migration with live_migration_wait_for_vif_plug=True.
-        if ((port['status'] == const.PORT_STATUS_DOWN and
-             ovn_const.MIGRATING_ATTR in port[portbindings.PROFILE].keys() and
-             port[portbindings.VIF_TYPE] == portbindings.VIF_TYPE_OVS)):
-            admin_context = n_context.get_admin_context()
-            LOG.info("Setting port %s status from DOWN to UP in order "
-                     "to emit vif-interface-plugged event.",
-                     port['id'])
-            self._plugin.update_port_status(admin_context, port['id'],
-                                            const.PORT_STATUS_ACTIVE)
-            # The revision has been changed. In the meantime
-            # port-update event already updated the OVN configuration,
-            # So there is no need to update it again here. Anyway it
-            # will fail that OVN has port with bigger revision.
-            return
-
         self._ovn_client.update_port(port, port_object=original_port)
         self._notify_dhcp_updated(port['id'])
 
@@ -564,7 +552,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         expected, and will not prevent the resource from being
         deleted.
         """
-        port = copy.deepcopy(context.current)
+        port = context.current
         port['network'] = context.network.current
         self._ovn_client.delete_port(port['id'], port_object=port)
 
@@ -750,7 +738,7 @@ class OVNMechanismDriver(api.MechanismDriver):
                 'NAT', nat['_uuid'],
                 ('external_mac', mac)).execute(check_error=True)
         else:
-            LOG.debug("Clearing up external_mac of port %s", port_id)
+            LOG.debug("Clearing up external_mac of port %s", port_id, mac)
             self._nb_ovn.db_clear(
                 'NAT', nat['_uuid'], 'external_mac').execute(check_error=True)
 
@@ -782,18 +770,12 @@ class OVNMechanismDriver(api.MechanismDriver):
             # becasue the router ports are unbind so, for OVN we are
             # forcing the status here. Maybe it's something that we can
             # change in core Neutron in the future.
-            db_port = ml2_db.get_port(admin_context, port_id)
-            if not db_port:
-                return
-
-            if db_port.device_owner in (const.DEVICE_OWNER_ROUTER_INTF,
-                                        const.DEVICE_OWNER_DVR_INTERFACE,
-                                        const.DEVICE_OWNER_ROUTER_HA_INTF):
+            port = self._plugin.get_port(admin_context, port_id)
+            if port.get('device_owner') in (const.DEVICE_OWNER_ROUTER_INTF,
+                                            const.DEVICE_OWNER_DVR_INTERFACE,
+                                            const.DEVICE_OWNER_ROUTER_HA_INTF):
                 self._plugin.update_port_status(admin_context, port_id,
                                                 const.PORT_STATUS_ACTIVE)
-            elif db_port.device_owner.startswith(
-                    const.DEVICE_OWNER_COMPUTE_PREFIX):
-                self._plugin.nova_notifier.notify_port_active_direct(db_port)
         except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
             LOG.debug('Port not found during OVN status up report: %s',
                       port_id)
@@ -808,21 +790,11 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._update_dnat_entry_if_needed(port_id, False)
         admin_context = n_context.get_admin_context()
         try:
-            db_port = ml2_db.get_port(admin_context, port_id)
-            if not db_port:
-                return
-
-            self._insert_port_provisioning_block(admin_context, port_id)
-            self._plugin.update_port_status(admin_context, port_id,
+            port = self._plugin.get_port(admin_context, port_id)
+            self._insert_port_provisioning_block(admin_context, port)
+            self._plugin.update_port_status(admin_context,
+                                            port['id'],
                                             const.PORT_STATUS_DOWN)
-
-            if db_port.device_owner.startswith(
-                    const.DEVICE_OWNER_COMPUTE_PREFIX):
-                self._plugin.nova_notifier.record_port_status_changed(
-                    db_port, const.PORT_STATUS_ACTIVE, const.PORT_STATUS_DOWN,
-                    None)
-                self._plugin.nova_notifier.send_port_status(
-                    None, None, db_port)
         except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
             LOG.debug("Port not found during OVN status down report: %s",
                       port_id)
@@ -858,8 +830,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         """Wait for metadata service to be provisioned.
 
         Wait until metadata service has been setup for this port in the chassis
-        it resides. If metadata is disabled or DHCP is not enabled for its
-        subnets, this function will return right away.
+        it resides. If metadata is disabled, this function will return right
+        away.
         """
         if config.is_ovn_metadata_enabled() and self._sb_ovn:
             # Wait until metadata service has been setup for this port in the
@@ -874,26 +846,6 @@ class OVNMechanismDriver(api.MechanismDriver):
                 LOG.warning("Logical port %s is not bound to a "
                             "chassis", port_id)
                 return
-
-            # Check if the port belongs to some IPv4 subnet with DHCP enabled.
-            context = n_context.get_admin_context()
-            port = self._plugin.get_port(context, port_id)
-            port_subnet_ids = set(
-                ip['subnet_id'] for ip in port['fixed_ips'] if
-                n_utils.get_ip_version(ip['ip_address']) == const.IP_VERSION_4)
-            if not port_subnet_ids:
-                # The port doesn't belong to any IPv4 subnet
-                return
-
-            subnets = self._plugin.get_subnets(context, filters=dict(
-                network_id=[port['network_id']], ip_version=[4],
-                enable_dhcp=True))
-
-            subnet_ids = set(
-                s['id'] for s in subnets if s['id'] in port_subnet_ids)
-            if not subnet_ids:
-                return
-
             try:
                 n_utils.wait_until_true(
                     lambda: datapath in
@@ -908,3 +860,91 @@ class OVNMechanismDriver(api.MechanismDriver):
                 LOG.warning("Metadata service is not ready for port %s, check"
                             " networking-ovn-metadata-agent status/logs.",
                             port_id)
+
+    def agent_from_chassis(self, chassis):
+        agent_type = ovn_const.OVN_CONTROLLER_AGENT
+        if ('enable-chassis-as-gw' in
+                chassis.external_ids.get('ovn-cms-options', [])):
+            agent_type = ovn_const.OVN_CONTROLLER_GW_AGENT
+        alive = self._nb_ovn.nb_global.nb_cfg == chassis.nb_cfg
+        return {
+            'binary': "ovn-controller",
+            'host': chassis.hostname,
+            'heartbeat_timestamp': datetime.datetime.utcnow(),
+            'availability_zone': 'n/a',
+            'topic': 'n/a',
+            'description':
+                chassis.external_ids.get(ovn_const.OVN_AGENT_DESC_KEY, ''),
+            'configurations': {
+                'chassis_name': chassis.name,
+                'bridge-mappings':
+                    chassis.external_ids.get('ovn-bridge-mappings', '')},
+            'start_flag': True,
+            'agent_type': agent_type,
+            'id': str(chassis.uuid),
+            'alive': alive,
+            'admin_state_up': True}
+
+    def patch_plugin_merge(self, method_name, new_fn, op=operator.add):
+        old_method = getattr(self._plugin, method_name)
+
+        @functools.wraps(old_method)
+        def fn(slf, *args, **kwargs):
+            new_method = types.MethodType(new_fn, self._plugin)
+            results = old_method(*args, **kwargs)
+            return op(results, new_method(*args, _driver=self, **kwargs))
+
+        setattr(self._plugin, method_name, types.MethodType(fn, self._plugin))
+
+    def patch_plugin_choose(self, method_name, new_fn):
+        old_method = getattr(self._plugin, method_name)
+
+        @functools.wraps(old_method)
+        def fn(slf, *args, **kwargs):
+            new_method = types.MethodType(new_fn, self._plugin)
+            try:
+                return new_method(*args, _driver=self, **kwargs)
+            except KeyError:
+                return old_method(*args, **kwargs)
+
+        setattr(self._plugin, method_name, types.MethodType(fn, self._plugin))
+
+
+def get_agents(self, context, filters=None, fields=None, _driver=None):
+    filters = filters or {}
+    agents = []
+    for c in _driver._sb_ovn.tables['Chassis'].rows.values():
+        a = _driver.agent_from_chassis(c)
+        if all(a[k] in v for k, v in filters.items()):
+            agents.append(a)
+    return agents
+
+
+def get_agent(self, context, id, fields=None, _driver=None):
+    c = _driver._sb_ovn.tables['Chassis'].rows[uuid.UUID(id)]
+    return _driver.agent_from_chassis(c)
+
+
+def update_agent(self, context, id, agent, _driver=None):
+    chassis = _driver._sb_ovn.tables['Chassis'].rows[uuid.UUID(id)]
+    agent = agent['agent']
+    # neutron-client always passes admin_state_up, openstack client doesn't
+    # and we can just fall through to raising in the case that admin_state_up
+    # is being set to False, otherwise the end-state will be fine
+    if not agent.get('admin_state_up', True):
+        pass
+    elif 'description' in agent:
+        _driver._sb_ovn.set_chassis_neutron_description(
+            chassis.name, agent['description']).execute(check_error=True)
+        return _driver.agent_from_chassis(chassis)
+    else:
+        # admin_state_up=True w/o description
+        return _driver.agent_from_chassis(chassis)
+    raise n_exc.BadRequest(resource='agent',
+                           msg='OVN agent status cannot be updated')
+
+
+def delete_agent(self, context, id, _driver=None):
+    c = _driver._sb_ovn.tables['Chassis'].rows[uuid.UUID(id)]  # noqa
+    raise n_exc.BadRequest(resource='agent',
+                           msg='OVN agents cannot be deleted')
