@@ -39,6 +39,11 @@ from networking_ovn.ovsdb import ovsdb_monitor
 LOG = log.getLogger(__name__)
 
 
+class OvnNbTransaction(idl_trans.Transaction):
+    def pre_commit(self, txn):
+        self.api.nb_global.increment('nb_cfg')
+
+
 # This version of Backend doesn't use a class variable for ovsdb_connection
 # and therefor allows networking-ovn to manage connection scope on its own
 class Backend(ovs_idl.Backend):
@@ -79,6 +84,24 @@ class Backend(ovs_idl.Backend):
             self, self.ovsdb_connection, self.ovsdb_connection.timeout,
             check_error, log_errors)
 
+    # Check for a column match in the table. If not found do a retry with
+    # a stop delay of 10 secs. This function would be useful if the caller
+    # wants to verify for the presence of a particular row in the table
+    # with the column match before doing any transaction.
+    # Eg. We can check if Logical_Switch row is present before adding a
+    # logical switch port to it.
+    @tenacity.retry(retry=tenacity.retry_if_exception_type(RuntimeError),
+                    wait=tenacity.wait_exponential(),
+                    stop=tenacity.stop_after_delay(10),
+                    reraise=True)
+    def check_for_row_by_value_and_retry(self, table, column, match):
+        try:
+            idlutils.row_by_value(self.idl, table, column, match)
+        except idlutils.RowNotFound:
+            msg = (_("%(match)s does not exist in %(column)s of %(table)s")
+                   % {'match': match, 'column': column, 'table': table})
+            raise RuntimeError(msg)
+
 
 class OvsdbConnectionUnavailable(n_exc.ServiceUnavailable):
     message = _("OVS database connection to %(db_schema)s failed with error: "
@@ -127,6 +150,15 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         self.idl._session.reconnect.set_probe_interval(
             cfg.get_ovn_ovsdb_probe_interval())
 
+    @property
+    def nb_global(self):
+        return next(iter(self.tables['NB_Global'].rows.values()))
+
+    def create_transaction(self, check_error=False, log_errors=True):
+        return OvnNbTransaction(
+            self, self.ovsdb_connection, self.ovsdb_connection.timeout,
+            check_error, log_errors)
+
     @contextlib.contextmanager
     def transaction(self, *args, **kwargs):
         """A wrapper on the ovsdbapp transaction to work with revisions.
@@ -578,24 +610,6 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
 
         return (ls, None)
 
-    # Check for a column match in the table. If not found do a retry with
-    # a stop delay of 10 secs. This function would be useful if the caller
-    # wants to verify for the presence of a particular row in the table
-    # with the column match before doing any transaction.
-    # Eg. We can check if Logical_Switch row is present before adding a
-    # logical switch port to it.
-    @tenacity.retry(retry=tenacity.retry_if_exception_type(RuntimeError),
-                    wait=tenacity.wait_exponential(),
-                    stop=tenacity.stop_after_delay(10),
-                    reraise=True)
-    def check_for_row_by_value_and_retry(self, table, column, match):
-        try:
-            idlutils.row_by_value(self.idl, table, column, match)
-        except idlutils.RowNotFound:
-            msg = (_("%(match)s does not exist in %(column)s of %(table)s")
-                   % {'match': match, 'column': column, 'table': table})
-            raise RuntimeError(msg)
-
     def get_floatingip(self, fip_id):
         # TODO(dalvarez): remove this check once the minimum OVS required
         # version contains the column (when OVS 2.8.2 is released).
@@ -657,68 +671,10 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def get_port_group(self, pg_name):
         if uuidutils.is_uuid_like(pg_name):
             pg_name = utils.ovn_port_group_name(pg_name)
-        try:
-            for pg in self._tables['Port_Group'].rows.values():
-                if pg.name == pg_name:
-                    return pg
-        except KeyError:
-            # TODO(dalvarez): This except block is added for backwards compat
-            # with old OVN schemas (<=2.9) where Port Groups are not present.
-            # This (and other conditional code around this feature) shall be
-            # removed at some point.
-            return
 
-    def pg_add(self, name=None, may_exist=False, **columns):
-        return cmd.PgAddCommand(self, name, may_exist=may_exist, **columns)
-
-    def pg_del(self, name, if_exists=False):
-        return cmd.PgDelCommand(self, name, if_exists=if_exists)
-
-    def pg_add_ports(self, pg_id, lsp):
-        return cmd.PgAddPortCommand(self, pg_id, lsp=lsp)
-
-    def pg_del_ports(self, pg_id, lsp, if_exists=False):
-        return cmd.PgDelPortCommand(self, pg_id, lsp=lsp, if_exists=if_exists)
-
-    def pg_acl_add(self, port_group, direction, priority, match, action,
-                   log=False, may_exist=False, **external_ids):
-        return cmd.PgAclAddCommand(self, port_group, direction, priority,
-                                   match, action, log, may_exist,
-                                   **external_ids)
-
-    def pg_acl_del(self, port_group, direction=None, priority=None,
-                   match=None):
-        return cmd.PgAclDelCommand(self, port_group, direction, priority,
-                                   match)
-
-    def pg_acl_list(self, port_group):
-        return cmd.PgAclListCommand(self, port_group)
-
-    def pg_get(self, pg):
-        return cmd.PgGetCommand(self, pg)
-
-    def get_port_groups(self):
-        port_groups = {}
-        try:
-            for row in self._tables['Port_Group'].rows.values():
-                name = getattr(row, 'name')
-                if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
-                   name == ovn_const.OVN_DROP_PORT_GROUP_NAME):
-                    continue
-                data = {}
-                for row_key in getattr(row, "_data", {}):
-                    data[row_key] = getattr(row, row_key)
-                port_groups[name] = data
-        except KeyError:
-            # TODO(dalvarez): This except block is added for backwards compat
-            # with old OVN schemas (<=2.9) where Port Groups are not present.
-            # This (and other conditional code around this feature) shall be
-            # removed at some point.
-            pass
-        return port_groups
-
-    def check_liveness(self):
-        return cmd.CheckLivenessCommand(self)
+        for pg in self._tables['Port_Group'].rows.values():
+            if pg.name == pg_name:
+                return pg
 
 
 class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
@@ -799,6 +755,11 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
             self, chassis, {'neutron-metadata-proxy-networks': nets},
             if_exists=True)
 
+    def set_chassis_neutron_description(self, chassis, description):
+        return cmd.UpdateChassisExtIdsCommand(
+            self, chassis, {ovn_const.OVN_AGENT_DESC_KEY: description},
+            if_exists=False)
+
     def get_network_port_bindings_by_ip(self, network, ip_address):
         rows = self.db_list_rows('Port_Binding').execute(check_error=True)
         # TODO(twilson) It would be useful to have a db_find that takes a
