@@ -120,10 +120,6 @@ class FakeOvsdbNbOvnIdl(object):
         self.get_lrouter.return_value = None
         self.delete_lrouter_ext_gw = mock.Mock()
         self.delete_lrouter_ext_gw.return_value = None
-        self.lsp_get_up = mock.Mock()
-        self.db_find = mock.Mock()
-        self.db_set = mock.Mock()
-        self.db_clear = mock.Mock()
         self.is_port_groups_supported = mock.Mock()
         # TODO(lucasagomes): Flip this return value to True at some point,
         # port groups should be the default method used by networking-ovn
@@ -138,9 +134,6 @@ class FakeOvsdbNbOvnIdl(object):
         self.pg_add_ports = mock.Mock()
         self.pg_del_ports = mock.Mock()
         self.lsp_get_up = mock.Mock()
-        self.nb_global = mock.Mock()
-        self.db_list_rows = mock.Mock()
-        self.lsp_list = mock.MagicMock()
 
 
 class FakeOvsdbSbOvnIdl(object):
@@ -154,9 +147,6 @@ class FakeOvsdbSbOvnIdl(object):
         self.get_chassis_data_for_ml2_bind_port = mock.Mock()
         self.get_chassis_data_for_ml2_bind_port.return_value = \
             ('fake', '', ['fake-physnet'])
-        self.get_logical_port_chassis_and_datapath = mock.Mock()
-        self.get_logical_port_chassis_and_datapath.return_value = \
-            ('fake', 'fake-dp')
 
 
 class FakeOvsdbTransaction(object):
